# This Build.PL for Dist-Zilla-Plugin-MetaResources-Template was generated by Dist::Zilla::Plugin::ModuleBuildTiny 0.014.
use strict;
use warnings;

use 5.006;
use Module::Build::Tiny 0.034;
Build_PL();
